@@ -19,6 +19,7 @@
 
 #include "mm-serial-parsers.h"
 #include "mm-errors.h"
+#include "mm-log.h"
 
 /* Clean up the response by removing control characters like <CR><LF> etc */
 static void
@@ -33,6 +34,13 @@ response_clean (GString *response)
         s -= 2;
     }
 
+    /* Contains duplicate '<CR><CR>' */
+    s = response->str;
+    while ((response->len >= 2) && (*s == '\r') && (*(s + 1) == '\r')) {
+        g_string_erase (response, 0, 1);
+        s = response->str;
+    }
+
     /* Starts with one or more '<CR><LF>' */
     s = response->str;
     while ((response->len >= 2) && (*s == '\r') && (*(s + 1) == '\n')) {
@@ -167,7 +175,7 @@ mm_serial_parser_v0_parse (gpointer data,
         response_clean (response);
 
     if (local_error) {
-        g_debug ("Got failure code %d: %s", local_error->code, local_error->message);
+        mm_dbg ("Got failure code %d: %s", local_error->code, local_error->message);
         g_propagate_error (error, local_error);
     }
 
@@ -188,12 +196,18 @@ mm_serial_parser_v0_destroy (gpointer data)
 }
 
 typedef struct {
+    /* Regular expressions for successful replies */
     GRegex *regex_ok;
     GRegex *regex_connect;
+    GRegex *regex_custom_successful;
+    /* Regular expressions for error replies */
     GRegex *regex_cme_error;
+    GRegex *regex_cms_error;
     GRegex *regex_cme_error_str;
+    GRegex *regex_ezx_error;
     GRegex *regex_unknown_error;
     GRegex *regex_connect_failed;
+    GRegex *regex_custom_error;
 } MMSerialParserV1;
 
 gpointer
@@ -207,13 +221,36 @@ mm_serial_parser_v1_new (void)
     parser->regex_ok = g_regex_new ("\\r\\nOK(\\r\\n)+$", flags, 0, NULL);
     parser->regex_connect = g_regex_new ("\\r\\nCONNECT.*\\r\\n", flags, 0, NULL);
     parser->regex_cme_error = g_regex_new ("\\r\\n\\+CME ERROR: (\\d+)\\r\\n$", flags, 0, NULL);
+    parser->regex_cms_error = g_regex_new ("\\r\\n\\+CMS ERROR: (\\d+)\\r\\n$", flags, 0, NULL);
     parser->regex_cme_error_str = g_regex_new ("\\r\\n\\+CME ERROR: ([^\\n\\r]+)\\r\\n$", flags, 0, NULL);
+    parser->regex_ezx_error = g_regex_new ("\\r\\n\\MODEM ERROR: (\\d+)\\r\\n$", flags, 0, NULL);
     parser->regex_unknown_error = g_regex_new ("\\r\\n(ERROR)|(COMMAND NOT SUPPORT)\\r\\n$", flags, 0, NULL);
     parser->regex_connect_failed = g_regex_new ("\\r\\n(NO CARRIER)|(BUSY)|(NO ANSWER)|(NO DIALTONE)\\r\\n$", flags, 0, NULL);
 
+    parser->regex_custom_successful = NULL;
+    parser->regex_custom_error = NULL;
+
     return parser;
 }
 
+void
+mm_serial_parser_v1_set_custom_regex (gpointer data,
+                                      GRegex *successful,
+                                      GRegex *error)
+{
+    MMSerialParserV1 *parser = (MMSerialParserV1 *) data;
+
+    g_return_if_fail (parser != NULL);
+
+    if (parser->regex_custom_successful)
+        g_regex_unref (parser->regex_custom_successful);
+    if (parser->regex_custom_error)
+        g_regex_unref (parser->regex_custom_error);
+
+    parser->regex_custom_successful = successful ? g_regex_ref (successful) : NULL;
+    parser->regex_custom_error = error ? g_regex_ref (error) : NULL;
+}
+
 gboolean
 mm_serial_parser_v1_parse (gpointer data,
                            GString *response,
@@ -234,11 +271,24 @@ mm_serial_parser_v1_parse (gpointer data,
 
     /* First, check for successful responses */
 
-    found = g_regex_match_full (parser->regex_ok, response->str, response->len, 0, 0, NULL, NULL);
-    if (found)
-        remove_matches (parser->regex_ok, response);
-    else
-        found = g_regex_match_full (parser->regex_connect, response->str, response->len, 0, 0, NULL, NULL);
+    /* Custom successful replies first, if any */
+    if (parser->regex_custom_successful) {
+        found = g_regex_match_full (parser->regex_custom_successful,
+                                    response->str, response->len,
+                                    0, 0, NULL, NULL);
+    }
+
+    if (!found) {
+        found = g_regex_match_full (parser->regex_ok,
+                                    response->str, response->len,
+                                    0, 0, NULL, NULL);
+        if (found)
+            remove_matches (parser->regex_ok, response);
+        else
+            found = g_regex_match_full (parser->regex_connect,
+                                        response->str, response->len,
+                                        0, 0, NULL, NULL);
+    }
 
     if (found) {
         response_clean (response);
@@ -247,6 +297,21 @@ mm_serial_parser_v1_parse (gpointer data,
 
     /* Now failures */
 
+    /* Custom error matches first, if any */
+    if (parser->regex_custom_error) {
+        found = g_regex_match_full (parser->regex_custom_error,
+                                    response->str, response->len,
+                                    0, 0, &match_info, NULL);
+        if (found) {
+            str = g_match_info_fetch (match_info, 1);
+            g_assert (str);
+            local_error = mm_mobile_error_for_code (atoi (str));
+            g_free (str);
+            g_match_info_free (match_info);
+            goto done;
+        }
+    }
+
     /* Numeric CME errors */
     found = g_regex_match_full (parser->regex_cme_error,
                                 response->str, response->len,
@@ -260,6 +325,23 @@ mm_serial_parser_v1_parse (gpointer data,
         goto done;
     }
 
+    /* Numeric CMS errors */
+    /* Todo
+     * One should probably add message service
+     * errors explicitly in mm-errors.h/c
+     */
+    found = g_regex_match_full (parser->regex_cms_error,
+                                response->str, response->len,
+                                0, 0, &match_info, NULL);
+    if (found) {
+        str = g_match_info_fetch (match_info, 1);
+        g_assert (str);
+        local_error = mm_mobile_error_for_code (atoi (str));
+        g_free (str);
+        g_match_info_free (match_info);
+        goto done;
+    }
+
     /* String CME errors */
     found = g_regex_match_full (parser->regex_cme_error_str,
                                 response->str, response->len,
@@ -273,6 +355,19 @@ mm_serial_parser_v1_parse (gpointer data,
         goto done;
     }
 
+    /* Motorola EZX errors */
+    found = g_regex_match_full (parser->regex_ezx_error,
+                                response->str, response->len,
+                                0, 0, &match_info, NULL);
+    if (found) {
+        str = g_match_info_fetch (match_info, 1);
+        g_assert (str);
+        local_error = mm_mobile_error_for_code (MM_MOBILE_ERROR_UNKNOWN);
+        g_free (str);
+        g_match_info_free (match_info);
+        goto done;
+    }
+
     /* Last resort; unknown error */
     found = g_regex_match_full (parser->regex_unknown_error,
                                 response->str, response->len,
@@ -314,7 +409,7 @@ done:
         response_clean (response);
 
     if (local_error) {
-        g_debug ("Got failure code %d: %s", local_error->code, local_error->message);
+        mm_dbg ("Got failure code %d: %s", local_error->code, local_error->message);
         g_propagate_error (error, local_error);
     }
 
@@ -332,9 +427,15 @@ mm_serial_parser_v1_destroy (gpointer data)
     g_regex_unref (parser->regex_connect);
     g_regex_unref (parser->regex_cme_error);
     g_regex_unref (parser->regex_cme_error_str);
+    g_regex_unref (parser->regex_ezx_error);
     g_regex_unref (parser->regex_unknown_error);
     g_regex_unref (parser->regex_connect_failed);
 
+    if (parser->regex_custom_successful)
+        g_regex_unref (parser->regex_custom_successful);
+    if (parser->regex_custom_error)
+        g_regex_unref (parser->regex_custom_error);
+
     g_slice_free (MMSerialParserV1, data);
 }
 
